/*
 * Copyright (c) 2002 Jan Harkes <jaharkes(at)cs.cmu.edu>
 * This code is distributed "AS IS" without warranty of any kind under the
 * terms of the GNU General Public License Version 2.
 */

#ifndef _GPSAPP_H_
#define _GPSAPP_H_

#include <sys/time.h>

/* configuration settings */
extern int show_scale;
extern int show_gpscoords;
extern int show_rubberband;
extern int show_metric;
extern int show_popups;
extern int show_abs;
extern int show_time;

/* screen coordinates */
struct xy { int x, y; };

/* geodetic coordinates (radians, WGS85 datum) */
struct coord {
    double lat, lon;
    struct xy xy;
};

/* route waypoints */
struct wp { int idx; char *desc; };

struct route {
    int	npts;
    int	nwps;
    struct xy *pts;
    struct wp *wps;
    int *dists;
};

extern int h0;
extern int do_refresh;
extern struct coord coord_center;

/* conversion functions (convert.c) */
double degtorad(const double deg);
double intdegtorad(const int deg);
double radtodeg(const double rad);
char *formatdist(const unsigned int distance);
char *time_estimate(const unsigned int distance);

void toTM(const struct coord *point, const struct coord *center, struct xy *xy);
long long distance2(const struct xy *coord1, const struct xy *coord2);
int bearing(const struct xy *coord1, const struct xy *coord2);
int towards(const struct xy *here, const struct xy *coord, const int dir);

/* screen update functions (draw.c) */
void draw_clear(void);
void draw_zoom(int inout);
void draw_point(const struct xy *coord, const int shade);
void draw_line(const struct xy *from, const struct xy *to, const int shade);
void draw_lines(const struct xy *pts, const int npts, const int shade);
void draw_msg(const char *msg);
void _draw_mark(const int x, const int y, const int dir, const int shade);
void draw_mark(const struct xy *pnt, const int dir, const int shade);
void draw_info(void);
void draw_scale(void);
void draw_gpscoords(void);
void draw_wpstext(void);
void draw_popup(char *text);
void draw_display(void);

/* tracking functions (track.c) */
void track_init(void);
void track_pos(void);
void track_draw(void);

/* route functions (route.c) */
extern int nextwp;
int routes_init(void);
void routes_list(void);
void route_select(int updown);
void route_load(void);
void route_init(void);
void route_locate(void);
void route_draw(struct xy *cur_pos);
int route_getwp(const int wp, struct xy *pos, unsigned int *dist, char **desc);

/* serial port functions (serial.c) */
void serial_open(void);
void serial_close(void);
void serial_poll(void);

/* nmea decoding (nmea.c) */
extern struct coord gps_coord;
extern int	    gps_bearing;
extern int	    gps_speed;
extern time_t       gps_time;
void nmea_decode(char *line, char xor);

#endif

